@@ -122,55 +122,8 @@ impl Md5 {
         let mut d0 = Wrapping(d0);
 
         for n in (0..buffer.len()).step_by(64) {
-            let mut a = a0;
-            let mut b = b0;
-            let mut c = c0;
-            let mut d = d0;
-
-            let chunk = &buffer[n..n + 64];
-            let m = [
-                Self::as_u32_le(&chunk[..4].try_into().unwrap()),
-                Self::as_u32_le(&chunk[4..8].try_into().unwrap()),
-                Self::as_u32_le(&chunk[8..12].try_into().unwrap()),
-                Self::as_u32_le(&chunk[12..16].try_into().unwrap()),
-                Self::as_u32_le(&chunk[16..20].try_into().unwrap()),
-                Self::as_u32_le(&chunk[20..24].try_into().unwrap()),
-                Self::as_u32_le(&chunk[24..28].try_into().unwrap()),
-                Self::as_u32_le(&chunk[28..32].try_into().unwrap()),
-                Self::as_u32_le(&chunk[32..36].try_into().unwrap()),
-                Self::as_u32_le(&chunk[36..40].try_into().unwrap()),
-                Self::as_u32_le(&chunk[40..44].try_into().unwrap()),
-                Self::as_u32_le(&chunk[44..48].try_into().unwrap()),
-                Self::as_u32_le(&chunk[48..52].try_into().unwrap()),
-                Self::as_u32_le(&chunk[52..56].try_into().unwrap()),
-                Self::as_u32_le(&chunk[56..60].try_into().unwrap()),
-                Self::as_u32_le(&chunk[60..64].try_into().unwrap()),
-            ];
-
-            for i in 0..64 {
-                let mut f;
-                let g: u32;
-
-                if i < 16 {
-                    f = (b & c) | (!b & d);
-                    g = i;
-                } else if i < 32 {
-                    f = (d & b) | (!d & c);
-                    g = (5 * i + 1) % 16;
-                } else if i < 48 {
-                    f = b ^ c ^ d;
-                    g = (3 * i + 5) % 16;
-                } else {
-                    f = c ^ (b | !d);
-                    g = (7 * i) % 16;
-                }
-
-                f += a + Wrapping(m[g as usize]) + Wrapping(Self::PRECOMPUTED_TABLE[i as usize]);
-                a = d;
-                d = c;
-                c = b;
-                b += Wrapping(u32::rotate_left(f.0, Self::SHIFT_TABLE[i as usize]));
-            }
+            let (a, b, c, d) =
+                Md5::calculate_chunk(&buffer[n..n + 64].try_into().unwrap(), a0, b0, c0, d0);
 
             a0 += a;
             b0 += b;
@@ -178,10 +131,68 @@ impl Md5 {
             d0 += d;
         }
 
-        return ((a0.0.to_be() as u128) << 96)
-            + ((b0.0.to_be() as u128) << 64)
-            + ((c0.0.to_be() as u128) << 32)
-            + d0.0.to_be() as u128;
+        return Md5::construct_digest(a0.0, b0.0, c0.0, d0.0);
+    }
+
+    fn calculate_chunk(
+        chunk: &[u8; 64],
+        mut a: Wrapping<u32>,
+        mut b: Wrapping<u32>,
+        mut c: Wrapping<u32>,
+        mut d: Wrapping<u32>,
+    ) -> (Wrapping<u32>, Wrapping<u32>, Wrapping<u32>, Wrapping<u32>) {
+        let m = [
+            Wrapping(Self::as_u32_le(&chunk[..4].try_into().unwrap())),
+            Wrapping(Self::as_u32_le(&chunk[4..8].try_into().unwrap())),
+            Wrapping(Self::as_u32_le(&chunk[8..12].try_into().unwrap())),
+            Wrapping(Self::as_u32_le(&chunk[12..16].try_into().unwrap())),
+            Wrapping(Self::as_u32_le(&chunk[16..20].try_into().unwrap())),
+            Wrapping(Self::as_u32_le(&chunk[20..24].try_into().unwrap())),
+            Wrapping(Self::as_u32_le(&chunk[24..28].try_into().unwrap())),
+            Wrapping(Self::as_u32_le(&chunk[28..32].try_into().unwrap())),
+            Wrapping(Self::as_u32_le(&chunk[32..36].try_into().unwrap())),
+            Wrapping(Self::as_u32_le(&chunk[36..40].try_into().unwrap())),
+            Wrapping(Self::as_u32_le(&chunk[40..44].try_into().unwrap())),
+            Wrapping(Self::as_u32_le(&chunk[44..48].try_into().unwrap())),
+            Wrapping(Self::as_u32_le(&chunk[48..52].try_into().unwrap())),
+            Wrapping(Self::as_u32_le(&chunk[52..56].try_into().unwrap())),
+            Wrapping(Self::as_u32_le(&chunk[56..60].try_into().unwrap())),
+            Wrapping(Self::as_u32_le(&chunk[60..64].try_into().unwrap())),
+        ];
+
+        for i in 0..64 {
+            let mut f;
+            let g: u32;
+
+            if i < 16 {
+                f = (b & c) | (!b & d);
+                g = i;
+            } else if i < 32 {
+                f = (d & b) | (!d & c);
+                g = (5 * i + 1) % 16;
+            } else if i < 48 {
+                f = b ^ c ^ d;
+                g = (3 * i + 5) % 16;
+            } else {
+                f = c ^ (b | !d);
+                g = (7 * i) % 16;
+            }
+
+            f += a + m[g as usize] + Wrapping(Self::PRECOMPUTED_TABLE[i as usize]);
+            a = d;
+            d = c;
+            c = b;
+            b += Wrapping(u32::rotate_left(f.0, Self::SHIFT_TABLE[i as usize]));
+        }
+
+        return (a, b, c, d);
+    }
+
+    fn construct_digest(a0: u32, b0: u32, c0: u32, d0: u32) -> u128 {
+        ((a0.to_be() as u128) << 96)
+            + ((b0.to_be() as u128) << 64)
+            + ((c0.to_be() as u128) << 32)
+            + d0.to_be() as u128
     }
 
     fn preprocess(input: &[u8], original_length_in_bits: u64) -> Vec<u8> {
